--[[
    Here are all the stored variables in use and their nature (global, char).

char
    companionOrder
    lastCompanion
    lastViewed
    mountOrder
    position
    searchFilter
    version

profile
    buttonPadding
    columns
    fuBarScrollHeight
    opacity
    position
    recallOnDismount
    recallOnLanding
    recallOnRez
    recallOnZone
    rows (not in use)
    scale
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("ZooKeeper", true)

local options, fuBarOptions

-- registration of options
function ZooKeeper:SetupOptions()
    self.options = self:GetOptions()
    self.config = LibStub("AceConfig-3.0")
    self.optionsDialog = LibStub("AceConfigDialog-3.0")
    self.config:RegisterOptionsTable("ZooKeeper", self.options, {"zookeeper", "zk"})
    self.optionsFrame = self.optionsDialog:AddToBlizOptions("ZooKeeper", "ZooKeeper")
end


-- init of actual options table
function ZooKeeper:InitOptions()
    options = {
        type = 'group',
        args = {
            show = {
                name = L['ParamShowName'],
                desc = L['ParamShowDesc'],
                type = 'execute',
                guiHidden = true,
                handler = self,
                func = 'Show'
            },
            enable = {
                name = L['ParamEnableName'],
                desc = L['ParamEnableDesc'],
                type = 'execute',
                guiHidden = true,
                handler = self,
                func = 'EnableCmdLine'
            },
            disable = {
                name = L['ParamDisableName'],
                desc = L['ParamDisableDesc'],
                type = 'execute',
                guiHidden = true,
                handler = self,
                func = 'DisableCmdLine'
            },
            columns = {
                name = L['ParamColumnsName'],
                desc = L['ParamColumnsDesc'],
                type = 'range',
                min = 1,
                max = 20,
                step = 1,
                handler = self,
                set = 'SetColumns',
                get = 'GetColumns',
                width = full,
                order = 10
            },
            buttonPadding = {
                name = L['ParamButtonPaddingName'],
                desc = L['ParamButtonPaddingDesc'],
                type = 'range',
                min = 1,
                max = 20,
                step = 1,
                handler = self,
                set = 'SetButtonPadding',
                get = 'GetButtonPadding',
                width = full,
                order = 20
            },
            opacity = {
                name = L['ParamOpacityName'],
                desc = L['ParamOpacityDesc'],
                type = 'range',
                min = 0,
                max = 1,
                step = .05,
                handler = self,
                set = 'SetOpacity',
                get = 'GetOpacity',
                width = full,
                order = 40
            },
            scale = {
                name = L['ParamScaleName'],
                desc = L['ParamScaleDesc'],
                type = 'range',
                min = 0.3,
                max = 3.0,
                step = 0.1,
                handler = self,
                set = 'SetScale',
                get = 'GetScale',
                width = full,
                order = 50
            },
            fuBarScrollHeight = {
                name = L['ParamPanelScrollHeightName'],
                desc = L['ParamPanelScrollHeightDesc'],
                type = 'range',
                min = 5,
                max = 50,
                step = 1,
                handler = self,
                set = 'SetFuBarScrollHeight',
                get = 'GetFuBarScrollHeight',
                width = full,
                order = 55
            },
            autoRecall = {
                name = L['ParamAutoRecallName'],
                type = 'header',
                order = 69
            },
            autoRecallOnLanding = {
                name = L['ParamAutoRecallOnLandingName'],
                desc = L['ParamAutoRecallOnLandingDesc'],
                type = 'toggle',
                handler = self,
                set = 'SetRecallOnLanding',
                get = 'GetRecallOnLanding',
                width = full,
                order = 70
            },
            autoRecallOnZone = {
                name = L['ParamAutoRecallOnZoneName'],
                desc = L['ParamAutoRecallOnZoneDesc'],
                type = 'toggle',
                handler = self,
                set = 'SetRecallOnZone',
                get = 'GetRecallOnZone',
                width = full,
                order = 80
            },
            autoRecallOnRez = {
                name = L['ParamAutoRecallOnRezName'],
                desc = L['ParamAutoRecallOnRezDesc'],
                type = 'toggle',
                handler = self,
                set = 'SetRecallOnRez',
                get = 'GetRecallOnRez',
                width = full,
                order = 90
            },
            autoRecallOnDismount = {
                name = L['ParamAutoRecallOnDismountName'],
                desc = L['ParamAutoRecallOnDismountDesc'],
                type = 'toggle',
                handler = self,
                set = 'SetRecallOnDismount',
                get = 'GetRecallOnDismount',
                width = full,
                order = 100
            },
            profile = {
                name = L['ParamProfileName'],
                type = 'header',
                cmdHidden = true,
                order = 400
            },
            saveAsProfile = {
                name = L['ParamSaveAsProfileName'],
                type = 'input',
                handler = self,
                set = 'SaveNewProfile',
                get = 'GetCurrentProfileName',
                cmdHidden = true,
                width = full,
                order = 410
            },
            loadProfile = {
                name = L['ParamLoadProfileName'],
                type = 'select',
                handler = self,
                set = 'LoadFromProfile',
                get = 'GetCurrentProfileName',
                values = 'GetProfileNames',
                cmdHidden = true,
                width = full,
                order = 420,
            }
         }
    }
end

function ZooKeeper:GetOptions()
    if not options then
        self:InitOptions()
    end
    return options
end

function ZooKeeper:GetColumns()
    return self.db.profile.columns
end

function ZooKeeper:SetColumns(info, value)
    self.db.profile.columns = value
    self:Update()
end

function ZooKeeper:GetRows()
    return self.db.profile.rows
end

function ZooKeeper:SetRows(info, value)
    self.db.profile.rows = value
    self:Update()
end

function ZooKeeper:GetButtonPadding()
    return self.db.profile.buttonPadding
end

function ZooKeeper:SetButtonPadding(info, value)
    self.db.profile.buttonPadding = value
    self:Update()
end

function ZooKeeper:GetOpacity()
    return self.db.profile.opacity
end

function ZooKeeper:SetOpacity(info, value)
    self.db.profile.opacity = value
    self.frame:UpdateBackdrop()
end

function ZooKeeper:GetScale()
    return self.db.profile.scale
end

function ZooKeeper:SetScale(info, value)
    self.db.profile.scale = value
    self.frame:SetScale(self.db.profile.scale)
end

function ZooKeeper:GetFuBarScrollHeight()
    return self.db.profile.fuBarScrollHeight
end

function ZooKeeper:SetFuBarScrollHeight(info, value)
    self.db.profile.fuBarScrollHeight = value
    local zfFrame = self.zooFuFrame
    if zfFrame then
        zfFrame:CreateMenu()
    end
end

function ZooKeeper:GetRecallOnLanding()
    return self.db.profile.recallOnLanding
end

function ZooKeeper:SetRecallOnLanding(info, value)
    self.db.profile.recallOnLanding = value
end

function ZooKeeper:GetRecallOnZone()
    return self.db.profile.recallOnZone
end

function ZooKeeper:SetRecallOnZone(info, value)
    self.db.profile.recallOnZone = value
end

function ZooKeeper:GetRecallOnRez()
    return self.db.profile.recallOnRez
end

function ZooKeeper:SetRecallOnRez(info, value)
    self.db.profile.recallOnRez = value
end

function ZooKeeper:SetRecallOnDismount(info, value)
    self.db.profile.recallOnDismount = value
end

function ZooKeeper:GetRecallOnDismount()
    return self.db.profile.recallOnDismount
end

function ZooKeeper:SetNewProfileNameForSave(info, value)
    self.newProfileNameForSave = value
end

function ZooKeeper:GetCurrentProfileName()
    return self.db:GetCurrentProfile()
end

function ZooKeeper:SaveNewProfile(info, value)
    local oldProfileName = self.db:GetCurrentProfile()
    if value ~= oldProfileName then
        self.db:SetProfile(value)
        self.db:CopyProfile(oldProfileName)
    end
end

function ZooKeeper:GetProfileNames()
    local profileNames, count = self.db:GetProfiles()
    return profileNames
end

function ZooKeeper:LoadFromProfile(info, value)
    local profileNames, count = self.db:GetProfiles()
    local newProfileName = profileNames[value]
    local oldProfileName = self.db:GetCurrentProfile()
    if newProfileName ~= oldProfileName then
        self.db:SetProfile(newProfileName)
    end
end

function ZooKeeper:EnableCmdLine()
   self:OnEnable()
end

function ZooKeeper:DisableCmdLine()
   self:OnDisable()
end
